@@ -95,6 +95,8 @@ for types in $tests; do
 	esac
 	echo -ne "$types: $what: "
 	cmd=`echo $cmd | sed "s|ipset|$ipset 2>.foo.err|"`
+	# For the case: ipset list | ... | xargs -n1 ipset
+	cmd=`echo $cmd | sed "s|ipset|$ipset|"`
 	eval $cmd
 	r=$?
 	# echo $ret $r
